/** Provides a d.s. taht allows potentially infinite long list of integers
 *  @author Josh Hug
 */

public class IntList {
    int head;     /* similar to car of a list */
    IntList tail; /* similar to cdr of a list */

    public IntList(int head, IntList tail) {
        this.head = head;
        this.tail = tail;
    }

    /** Returns a new IntList containing the ints in ARGS. */
    public static IntList list(Integer ... args) {
        IntList result, p;

        if (args.length > 0) {
            result = new IntList(args[0], null);
        } else {
            return null;
        }

        int k;
        for (k = 1, p = result; k < args.length; k += 1, p = p.tail) {
            p.tail = new IntList(args[k], null);
        }
        return result;
    }

    /* increments P by x non-destructively, 
       returning a new list */
    public static IntList incrList(IntList P, int n) {
        if (P == null)
            return null;

        /* List is not null, so make a new list that
           is incremented */
        
        return new IntList(P.head + n, 
                           incrList(P.tail, n));
    }

    /** Returns true iff X is an IntList containing the same sequence of ints
     *  as THIS. Cannot handle IntLists with cycles. */
    public boolean equals(Object x) {
        if (!(x instanceof IntList)) {
            return false;
        }
        IntList L = (IntList) x;
        IntList p;
        
        for (p = this; p != null && L != null; p = p.tail, L = L.tail) {
            if (p.head != L.head) {
                return false;
            }
        }
        if (p != null || L != null) {
            return false;
        }
        return true;
    }

    /** 
     */

    public static void main(String[] args) {
        IntList P = IntList.list(0, 1, 2, 3);
        incrList(P, 3);
    }


} 